@@ -69,6 +69,11 @@ type (
 
 		// NOTE: FilterSubjects requires nats-server v2.10.0+
 		FilterSubjects []string `json:"filter_subjects,omitempty"`
+
+		// Metadata is additional metadata for the Consumer.
+		// Keys starting with `_nats` are reserved.
+		// NOTE: Metadata requires nats-server v2.10.0+
+		Metadata map[string]string `json:"metadata,omitempty"`
 	}
 
 	OrderedConsumerConfig struct {
