@@ -251,3 +251,66 @@ func TestKeyValueCreate(t *testing.T) {
 		t.Fatalf("Unexpected error code, got: %v", kerr.APIError().ErrorCode)
 	}
 }
+
+func TestKeyValueSourcing(t *testing.T) {
+	s := RunBasicJetStreamServer()
+	defer shutdownJSServerAndRemoveStorage(t, s)
+
+	nc, js := jsClient(t, s)
+	defer nc.Close()
+
+	kvA, err := js.CreateKeyValue(&KeyValueConfig{Bucket: "A"})
+	if err != nil {
+		t.Fatalf("Error creating kv: %v", err)
+	}
+
+	_, err = kvA.Create("keyA", []byte("1"))
+	if err != nil {
+		t.Fatalf("Error creating key: %v", err)
+	}
+
+	if _, err := kvA.Get("keyA"); err != nil {
+		t.Fatalf("Got error getting keyA from A: %v", err)
+	}
+
+	kvB, err := js.CreateKeyValue(&KeyValueConfig{Bucket: "B"})
+	if err != nil {
+		t.Fatalf("Error creating kv: %v", err)
+	}
+
+	_, err = kvB.Create("keyB", []byte("1"))
+	if err != nil {
+		t.Fatalf("Error creating key: %v", err)
+	}
+
+	kvC, err := js.CreateKeyValue(&KeyValueConfig{Bucket: "C", Sources: []*StreamSource{{Name: "A"}, {Name: "B"}}})
+	if err != nil {
+		t.Fatalf("Error creating kv: %v", err)
+	}
+
+	// Wait half a second to make sure it has time to populate the stream from it's sources
+	i := 0
+	for {
+		status, err := kvC.Status()
+		if err != nil {
+			t.Fatalf("Error getting bucket status: %v", err)
+		}
+		if status.Values() == 2 {
+			break
+		} else {
+			i++
+			if i > 3 {
+				t.Fatalf("Error sourcing bucket does not contain the expected number of values")
+			}
+		}
+		time.Sleep(20 * time.Millisecond)
+	}
+
+	if _, err := kvC.Get("keyA"); err != nil {
+		t.Fatalf("Got error getting keyA from C: %v", err)
+	}
+
+	if _, err := kvC.Get("keyB"); err != nil {
+		t.Fatalf("Got error getting keyB from C: %v", err)
+	}
+}
